﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Telerik.Windows;
using Telerik.Windows.Controls;

namespace GoldRuler
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        #region Fields
        private bool _isMouseLeftButtonDown = false;

        private Dock _currentPlacement = Dock.Left;

        private const int MIN_RULER_LENGTH = 550;

        private const int MIN_RULER_READING = 200;

        private const int MIN_RULER_HEIGHT = 80;

        private const double GOLDEN_RATIO = 1.618;

        private const int STARTPOINT_X = 0;
        private const int STARTPOINT_Y = 0;

        private double _currentReading;
        private bool _isLoaded = false; 
        #endregion

        public MainWindow()
        {
            InitializeComponent();

            RulerLength = MIN_RULER_LENGTH;

            XRuler.MouseMove += XRuler_MouseMove;
            XRuler.Placement = _currentPlacement;

            XRuler.PreviewMouseDown += XRuler_MouseLeftButtonDown;
            XRuler.PreviewMouseUp += XRuler_MouseLeftButtonUp;

            radContextMenu.PlacementTarget = this.XRuler;
            radContextMenu.ItemClick += radContextMenu_ItemClick;

            Reset();

            Loaded += MainWindow_Loaded;
        }


        #region Properties
        public double RulerLength
        {
            get { return (_currentPlacement == Dock.Top || _currentPlacement == Dock.Bottom) ? this.Width : this.Height; }
            set
            {
                double length = value > MIN_RULER_LENGTH ? value : MIN_RULER_LENGTH;

                switch (_currentPlacement)
                {
                    case Dock.Top:
                    case Dock.Bottom:
                        {
                            this.Width = length;
                            xDiagram.Width = length;
                            break;
                        }
                    default:
                        {
                            this.Height = length;
                            xDiagram.Height = length;
                            break;
                        }
                }
            }
        }

        public double RulerHeight
        {
            get { return (_currentPlacement == Dock.Top || _currentPlacement == Dock.Bottom) ? this.Height : this.Width; }
            set
            {
                double height = value > MIN_RULER_HEIGHT ? value : MIN_RULER_HEIGHT;

                switch (_currentPlacement)
                {
                    case Dock.Top:
                    case Dock.Bottom:
                        {
                            this.Height = height;
                            xDiagram.Height = height;
                            break;
                        }
                    default:
                        {
                            this.Width = height;
                            xDiagram.Width = height;
                            break;
                        }
                }
            }
        } 
        #endregion

        #region Event Handlers

        void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            ResetWindowPosition();

            _isLoaded = true;
        }

        void radContextMenu_ItemClick(object sender, RadRoutedEventArgs e)
        {
            if (e == null || e.Source == null || !(e.Source is RadMenuItem)) return;

            RadMenuItem item = e.Source as RadMenuItem;

            if (item.Header.ToString().ToLower() == "flip")
            {
                _currentReading = 0;
                Reset();
            }
            else if (item.Header.ToString().ToLower() == "close")
            {
                this.Close();
            }
        }

        void XRuler_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            _isMouseLeftButtonDown = false;
        }

        void XRuler_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            _isMouseLeftButtonDown = true;
        }


        void XRuler_MouseMove(object sender, MouseEventArgs e)
        {
            if (_isMouseLeftButtonDown)
            {
                try
                {
                    this.DragMove();
                }
                catch (Exception)
                {
                    _isMouseLeftButtonDown = false;
                }
            }
            else if (e != null)
            {
                Point point = e.GetPosition(XRuler);

                double currentReading = (_currentPlacement == Dock.Top || _currentPlacement == Dock.Bottom) ? point.X : point.Y;

                if (_currentReading == currentReading) return;

                // if current reading changed, get ruler length and reading

                ProcessCurrentReading(currentReading);
            }
        }

        #endregion


        #region Private methods
        private void ClearCurrentReadingDisplay()
        {
            txtGolden1.Text = string.Empty;
            txtGolden2.Text = string.Empty;
            txtReading.Text = string.Empty;

            Seperater1.Text = Seperater2.Text = string.Empty;
        }

        private void SetCurrentReadingDisplay(double reading, double golden1, double golden2)
        {
            txtReading.Text = reading.ToString();
            txtGolden1.Text = golden1.ToString();
            txtGolden2.Text = golden2.ToString();

            Seperater1.Text = Seperater2.Text = "  :  ";

            if (_currentPlacement == Dock.Top)
            {
                //Swithc to Left
                Grid.SetColumn(TxtReadBlock, 1);
                Grid.SetRow(TxtReadBlock, 1);

                TxtReadBlock.Width = MIN_RULER_LENGTH;
                TxtReadBlock.Margin = new Thickness(5, 0, 0, 0);
            }
            else if (_currentPlacement == Dock.Left)
            {
                //Swithc to Top
                Grid.SetColumn(TxtReadBlock, 0);
                Grid.SetRow(TxtReadBlock, 2);

                TxtReadBlock.Width = 6;

                TxtReadBlock.Margin = new Thickness(3, 0, 0, 0);
            }

        }

        /// <summary>
        /// Used to reset the control based on its current placement
        /// </summary>
        private void Reset()
        {
            ClearCurrentReadingDisplay();

            if (_currentPlacement == Dock.Top)
            {
                //Swithc to Left
                XRuler.Placement = _currentPlacement = Dock.Left;
                Grid.SetColumn(XRuler, 0);
                Grid.SetRow(XRuler, 2);
            }
            else if (_currentPlacement == Dock.Left)
            {
                //Swithc to Top
                XRuler.Placement = _currentPlacement = Dock.Top;
                Grid.SetColumn(XRuler, 1);
                Grid.SetRow(XRuler, 1);
            }

            Flip();

            if (_isLoaded) ResetWindowPosition();
        }



        /// <summary>
        /// Used to process the reading of the ruler as user moving mouse on the ruler
        /// </summary>
        /// <param name="currentReading"></param>
        private void ProcessCurrentReading(double currentReading)
        {
            if (currentReading > MIN_RULER_READING)
            {
                RulerLength = currentReading / 0.7;
            }
            else
            {
                RulerLength = MIN_RULER_LENGTH;
            }

            _currentReading = currentReading;

            //adjust the ruler length
            double y1 = currentReading / GOLDEN_RATIO;
            double y2 = currentReading * GOLDEN_RATIO;

            SetCurrentReadingDisplay(currentReading, Math.Round(y1, 3), Math.Round(y2, 3));

            UpdateGoldenLineLength(y1, y2);

        }

        private void UpdateGoldenLineLength(double y1, double y2)
        {
            switch (_currentPlacement)
            {
                case Dock.Top:
                    {
                        shorterGoldenLine.Position = new Point(STARTPOINT_X, STARTPOINT_Y);
                        shorterGoldenLine.Height = y1;
                        shorterGoldenLine.Width = 2;
                        longerGoldenLine.Position = new Point(STARTPOINT_X, shorterGoldenLine.Height);
                        longerGoldenLine.Height = y2 - shorterGoldenLine.Height;
                        longerGoldenLine.Width = 2;

                        break;
                    }
                case Dock.Left:
                    {
                        shorterGoldenLine.Position = new Point(STARTPOINT_X, STARTPOINT_Y);
                        shorterGoldenLine.Width = y1;
                        shorterGoldenLine.Height = 2;
                        longerGoldenLine.Position = new Point(shorterGoldenLine.Width, STARTPOINT_Y);
                        longerGoldenLine.Width = y2 - shorterGoldenLine.Width;
                        longerGoldenLine.Height = 2;

                        break;
                    }
                default:
                    {
                        break;
                    }
            }

            RulerHeight = y2 + MIN_RULER_HEIGHT;
        }

        /// <summary>
        /// Used to set up the start up location of the ruler
        /// </summary>
        private void ResetWindowPosition()
        {
            // Get absolute location on screen of upper left corner of button

            Point locationFromScreen = XRuler.PointToScreen(new Point(10, 10));

            // Transform screen point to WPF device independent point

            PresentationSource source = PresentationSource.FromVisual(this);

            System.Windows.Point targetPoints = source.CompositionTarget.TransformFromDevice.Transform(locationFromScreen);

            this.Left = targetPoints.X;
            this.Top = targetPoints.Y;
        }

        /// <summary>
        /// Used to reset the current control and diagram width and height after a flip click
        /// </summary>
        private void Flip()
        {
            double oldWidth = this.RulerLength;
            double oldHeight = this.RulerHeight;

            RulerHeight = oldWidth;
            RulerLength = oldHeight;

            UpdateGoldenLineLength(0, 0);
        } 
        #endregion
       
    }
}
